## Create identities for Fabric client components (registration and enrollment)

The `fabric-user-enrollment.js` script registers and enrolls users in the blockchain network and stores their certificates in either Cloudant or IBM Certificate Manager. 

### Prerequisites:
The Fabric identity registration and enrollment script, `fabric-user-enrollment.js`, relies on the following prerequisites:

1. A running IBM Blockchain Platform network instance on the IBM Cloud.
2. A running Cloudant service instance **or** a running IBM Certificate Manager service instance on the IBM Cloud.

### Running the script
1. Log on to IBM Cloud and access your IBP Blockchain Dashboard.
2. Download connection profile. For instructions on how to do that on the IBM Blockchain Platform, see [here](https://cloud.ibm.com/docs/services/blockchain/howto?topic=blockchain-ibp-console-app#ibp-console-app-profile).
3. Copy the file into the `scripts/fabric/config/connection-profile.json`.
	> Please assure that the `registrar` object with the admin credentials are included in the `connection-profile.json`. If they are not please append to the `certificateAuthorities` object within the unique CA IP address entry inside the `connection-profile.json` file. The object structure should look like this:

	```json
	{
	  "certificateAuthorities": {
	    "<CA IP address>": {
	      "registrar": [
		{
		  "enrollId": "<admin>",
		  "enrollSecret": "<adminpw>"
		}
	      ]
	    }
	  }
	}
	```
	* `admin` is bootstrap identity which you setup at the time CA creation which have values `enrollId` and `enrollSecret`.

4. Grab the credentials for the appropriate persistent store being used.
- For **Cloudant** as persistent store:

	- Copy to the clipboard the Cloudant connection credentials from the **corresponding** Cloudant dashboard; these credentials are found under the `Service credentials` section.

	- Paste the Cloudant credentials into the file `scripts/fabric/config/cloudant-credentials.json`.

- For **IBM Certificate Manager** as persistent store:
	- Update the file `certificate-manager-credentials.json` under the `scripts/fabric/config` folder, with the values for `url`, `apiKey`, and `instanceId`.
	> To determine the **region** (e.g. `us-south`, `eu-gb`, etc.) for your IBM Certificate Manager instance, please see the [IBM Cloud Certificate Manager API](https://cloud.ibm.com/apidocs/certificate-manager) documentation.

	```
	{
		"url": 'https://{region}.certificate-manager.cloud.ibm.com',
		"apiKey": "*****************",
		"instanceId":"***********************"
	}
	```
	 To obtain the `apiKey` and `instanceId` attributes, log on to the IBM Cloud:

	 - `apiKey` - For information on how to create an API Key for accessing IBM Certificate Manager, please see [Configuring IBM Certificate Manager As Persistent Store To Store The Fabric Credentials](https://github.ibm.com/BlockchainLabs/certificate-manager-wallet#configuring-ibm-certificate-manager-as-persistent-store-for-fabric-credentials). 
	 >Save the `apiKey` value when it is created since you **won't** be able to obtain it after that time.	  
	 
	 - `instanceId` - Log on to the IBM Cloud and access the **corresponding** IBM Certificate Manager instance and then click on `Settings` -> `Instance Info` and copy the `Service Instance CRN` value.

5. Create a `identity-config.json` file under the `scripts/fabric/config` folder. This file should contain the component identity information. As an example, see below or open the `scripts/fabric/config/sample-users.json` :

	```json 	
	{
	  "users": [
	    {
	      "org": "org1",
	      "user": "ryan@org1.com",
	      "pw": "ryan@org1",
	      "affiliation": "org1",
	      "role": "client",
	      "attrs": [
		{
		  "name": "orgType",
		  "value": "org1",
		  "ecert": true
		}
	      ]
	    }
	  ]
	}
	```

	* Ensure the values for the `org` and `affiliation` attributes are correct. Validate the values of these attributes with the `connection profile` file that you just downloaded.
	* `attrs` is an array of key/value attributes where `orgtype` is the type of the organization and can be `org1` or `org2`. `ecert` is an optional field; a value of `true` indicates that this attribute should be included in an enrollment certificate by default.

7. Execute the `fabric-user-enrollment.js` script as follows:
	```script
	npm run create-fabric-users -- --option=<value> --persistent=<value>
	```
 	- where the value of **option** can be `register` or `enroll`. If no option is passed, the default will perform both sequentially.

	- where the value of **persistent** can be `cloudant` or `certificate-manager`. If no store value is passed, the default store will be `certificate-manager`.


### Troubleshooting
1. Fabric-ca request register failed with errors [[{"code":20,"message":"Authorization failure"}]]

	First make sure the connection profile is correct. Also, note that this error appears when there has been a previous run of the script against a different connection profile. In such case, remove previously downloaded certificates from the project's root directory:
   	* Files ending with `-priv`
   	* Files ending with `-pub`
   	* Files with name of admin identity; generally this will be a file with name `admin`

2.  Registration of user failed with affiliation validation

	Ensure the affiliation section is populated correctly (inspect the connection profile file and the contents of the `identity-config.json` file).
